From a05fc4c6db50ec0c013e91c08b920e714c86a310 Mon Sep 17 00:00:00 2001
From: Tobias Jakobi <liquid.acid@gmx.net>
Date: Thu, 23 Apr 2009 00:38:45 +0200
Subject: [wined3d] shader_hw_sample: add NP2 fixup code

---
 dlls/wined3d/arb_program_shader.c |   24 +++++++++++++++++++-----
 1 files changed, 19 insertions(+), 5 deletions(-)

diff --git a/dlls/wined3d/arb_program_shader.c b/dlls/wined3d/arb_program_shader.c
index 92c6d62..2f1e633 100644
--- a/dlls/wined3d/arb_program_shader.c
+++ b/dlls/wined3d/arb_program_shader.c
@@ -695,6 +695,7 @@ static void shader_hw_sample(const struct wined3d_shader_instruction *ins, DWORD
     SHADER_BUFFER *buffer = ins->ctx->buffer;
     DWORD sampler_type = ins->ctx->reg_maps->samplers[sampler_idx] & WINED3DSP_TEXTURETYPE_MASK;
     const char *tex_type;
+    BOOL np2_fixup = FALSE;
     IWineD3DBaseShaderImpl *This = (IWineD3DBaseShaderImpl *)ins->ctx->shader;
     IWineD3DDeviceImpl *device = (IWineD3DDeviceImpl *) This->baseShader.device;
 
@@ -712,10 +713,13 @@ static void shader_hw_sample(const struct wined3d_shader_instruction *ins, DWORD
             }
             if (shader_is_pshader_version(ins->ctx->reg_maps->shader_version))
             {
-               const IWineD3DPixelShaderImpl* const ps = (const IWineD3DPixelShaderImpl*)This;
-               if(ps->cur_args->np2_fixup & (1 << sampler_idx)) {
-                   FIXME("NP2 texcoord fixup is currently not implemented in ARB mode (use GLSL instead).\n");
-               }
+               const IWineD3DPixelShaderImpl* const ps = (const IWineD3DPixelShaderImpl *)This;
+               if(ps->cur_np2fixup->active & (1 << sampler_idx)) {
+                   np2_fixup = TRUE;
+                   if(bias || projected) {
+                       FIXME("Projected or biased sampling from NP2 textures is unsupported.\n");
+                   }
+                }
             }
             break;
 
@@ -740,7 +744,17 @@ static void shader_hw_sample(const struct wined3d_shader_instruction *ins, DWORD
     } else if (projected) {
         shader_addline(buffer, "TXP %s, %s, texture[%u], %s;\n", dst_str, coord_reg, sampler_idx, tex_type);
     } else {
-        shader_addline(buffer, "TEX %s, %s, texture[%u], %s;\n", dst_str, coord_reg, sampler_idx, tex_type);
+        if (np2_fixup) {
+            const IWineD3DPixelShaderImpl* const ps = (const IWineD3DPixelShaderImpl *)This;
+            const WORD idx = ps->cur_np2fixup->idx[sampler_idx];
+
+            shader_addline(buffer, "MUL TMP, np2fixup[%u].%s, %s;\n", idx >> 1,
+                           (idx % 2) ? "zwxy" : "xyzw", coord_reg);
+
+            shader_addline(buffer, "TEX %s, TMP, texture[%u], %s;\n", dst_str, sampler_idx, tex_type);
+        } else {
+            shader_addline(buffer, "TEX %s, %s, texture[%u], %s;\n", dst_str, coord_reg, sampler_idx, tex_type);
+        }   
     }
 
     if (shader_is_pshader_version(ins->ctx->reg_maps->shader_version))
-- 
1.6.0.6

